'--------------------------------------------------------------
' AtriumUnum (RoomControl v5.0 Build 0015) by Dipl.-Ing. M.Sc. Christian Nding
'
' ATmega32/644 and ENC28J60 Webserver (optional)
'--------------------------------------------------------------
' Software based on Code by Ben Zijlstra, Viktor Varga, HansHans and Htti
'--------------------------------------------------------------
' Bascom 1.11.9.3
'
' Enable "optimize code" in the compiler settings
'
' Fuse Bits:  CKOPT set !
'--------------------------------------------------------------
' T6963c Grafik-LCD am an beiden Erweiterungsports des Boardes
' Touchscreen an den vier Analogeingngen des J3-Erweiterungsports
' PWM-Gesteuerte Helligkeit des LCDs ber PortB.3 (Timer0-PWM)
' DS18S20 Temperaturfhler an PortB.0
'--------------------------------------------------------------

'*****************************************************************************************************
'$regfile = "m32def.dat"
$regfile = "m644def.dat"
$crystal = 16000000
$baud = 250000
$hwstack = 160
$swstack = 160
$framesize = 180

'Einstellungen
Const Network = 1                                           ' Aktiviert / Deaktiviert die Netzwerkfunktionen
Const Rc5enabled = 1                                        ' Aktiviert/Deaktiviert den RC5-Empfang
Const Rc5debug = 0                                          ' Zeigt empfangene RC5-Codes an
Const Analogclock = 1
Const Wachhund = 1                                          ' 0=Watchdog aus 1= Watchdog ein

'*****************************************************************************************************

$include "eeprom.inc"

'*****************************************************************************************************


#if Network = 1
   $include "enc28j60.inc"                                  'Konstanten fr ENC28J60
   $lib "tcpip.lbx"                                         'Fr Checksummen-Berechnung
#endif
$lib "datetime.lbx"                                         'Fr Date()- und Time()-Befehle

'Konstanten definieren
#if Network = 1
   Const Max_framelen = 1500                                'max. Lnge = 1518, davon gehen noch 58 fr Header etc. weg
   Const Max_ports = 1                                      'maximale Anzahl geffneter Verbindungen (Versuchsstadium)
   Const Lebenszeit_sekunden = 15                           'Lebenszeit bis Verbindung getrennt wird in sec.
#endif

' SoftClock einrichten
Const Reload = 62500                                        '28800 = 500ms 57600= 1s
Config Timer1 = Timer , Prescale = 256                      'fr 14,7456 Mhz
Ocr1ah = High(reload)
Ocr1al = Low(reload)
Tccr1a = 0
Set Tccr1b.3
On Compare1a Timer1_isr
Enable Compare1a                                            '

Config Date = Dmy , Separator = .

' Dimmerkernel einrichten
Config Timer2 = Timer , Prescale = 128 , Pwm = Off          '1 | 8 | 32 | 64 | 128 | 256 | 1024
On Timer2 Dimmerkernel
Timer2 = 125                                                ' Timer-Interrupt jede ms ausfhren (16000000MHz/128/125=1000kHz)
Enable Timer2

#if Network = 1
   'Netzwerkkarte einrichten
   Config Int2 = Falling
   On Int2 Isr_int2                                         'ISR fr ENC28J60 Netzwerk
#endif

'DMX512-Interrupt konfigurieren
On Utxc Dmxsend                                             'ISR fr DMX512-Transmitter
Enable Utxc

$include "variables.inc"

#if Network = 1
   $include "variables_network.inc"                         'do not move or modify this entry !!!!!
#endif

Dmxstate = 0                                                '0=Break, 1=Startb, 2=Data
Curdmxch = 0
Currentchannel = 1
Globalfadeintime = 3

#if Network = 1
   '*************** MAC und IP ********************************************************************
   'MAC address of the NET-IO
   Mymac(1) = &H00
   Mymac(2) = &H22
   Mymac(3) = &HF9
   Mymac(4) = &H01
   Mymac(5) = &H32
   Mymac(6) = &HDF

   'MAC address of your Router/Gateway , overwritten by DHCP
   Router_mac(1) = &H00
   Router_mac(2) = &H16
   Router_mac(3) = &H01
   Router_mac(4) = &HC7
   Router_mac(5) = &H65
   Router_mac(6) = &HF0

   'fix IP address of the NET-IO , overwritten by DHCP
   Myip(1) = 192
   Myip(2) = 168
   Myip(3) = 0
   Myip(4) = 100

   'fix IP address of the DNS Server , overwritten by DHCP
   Mydns(1) = 192
   Mydns(2) = 168
   Mydns(3) = 0
   Mydns(4) = 1
   '*****************************************************************************************************

   'Netzwerk
   Enc28j60_cs Alias Portb.4                                'CS pin of the ENC28J60
   Config Enc28j60_cs = Output
   Enable Int2                                              'enable interrupt 2

   'Pins
   Config Portb.1 = Input                                   'J11 to switch on DHCP
   Config Portb.2 = Input                                   'Interrupt Pin
   Porta = &B00001111
   Portb.1 = 1                                              'Pullup
   Portb.2 = 1                                              'Pullup
#endif

'Uart einrichten
Ucsrc = &B10001110                                          '8N2

'Temperatursensoren per 1-Wire-Verbindung
Config 1wire = Portb.0                                      'fr den DS1820

'RC5-Receiver
#if Rc5enabled = 1
   Config Rc5 = Pina.7                                      'RC5 Code nutzt Timer0

   $asm                                                     ' Wichtig, damit der Getrc5 bei neueren ATmegas mit Timer0 funktioniert!
   .equ Timsk = Timsk0
   $end Asm
#endif

'ADC
Config Adc = Single , Prescaler = Auto                      ' Fr Helligkeit
Start Adc

'Watchdog
#if Wachhund = 1
   Config Watchdog = 4096
   Start Watchdog
   Reset Watchdog
#endif

' Hintergrundbeleuchtung
Config Portb.3 = Output
Backlight Alias Portb.3
Set Backlight                                               ' Hintergrundbeleuchtung einschalten

'Grafikdisplay T6963
Config Graphlcd = 240 * 128 , Dataport = Portc , Controlport = Portd , Ce = 4 , Cd = 5 , Wr = 2 , Rd = 3 , Reset = 6 , Fs = 7 , Mode = 6
Cls
Cursor Off
Wait 1

#if Wachhund = 1
   Reset Watchdog
#endif

#if Network = 1
   'SPI-Bus einrichten
   Config Spi = Hard , Interrupt = Off , Data Order = Msb , Master = Yes , Polarity = Low , Phase = 0 , Clockrate = 4 , Noss = 1 , Spiin = 0
   Spiinit
#endif




'****************************** INITIALISIERUNG ******************************
'Bootscreen anzeigen
Cls
Showpic 0 , 0 , Picstartup
Wait 1                                                      'Splashscreen etwas lnger anzeigen
#if Wachhund = 1
   Reset Watchdog
#endif
Wait 1                                                      'Splashscreen etwas lnger anzeigen

Locate 1 , 1
Lcd "AtriumUnum (RoomControl v5.0 Build 0015)"

#if Network = 1
   Locate 2 , 1
   Lcd "Starte ENC28J60-Netzwerk..."

   'Reset the Enc28J60
   Call Enc28j60_reset
   Call Enc28j60_setextclock

   Locate 2 , 29
   Lcd "(v" ; Enc28j60_version() ; ")"
   Call Enc28j60_init

   'LEDs blinken lassen
   'Call Enc28j60writephyword(phlcon , &H0AA2)
   'Call Enc28j60writephyword(phlcon , &H0472)                  'RX+TX ACT + LINK
   'Call Enc28j60writephyword(phlcon , &H01D8)                  'RX+TX ACT + LINK
   Call Enc28j60writephyword(phlcon , &H347A)               'RX+TX ACT + LINK

   Locate 3 , 1
   Lcd "IP-Adresse: " ; Myip(1) ; "." ; Myip(2) ; "." ; Myip(3) ; "." ; Myip(4) ; " (Statisch)"
   Locate 4 , 1
   Lcd "MAC-Adresse: " ; Hex(mymac(1)) ; "-" ; Hex(mymac(2)) ; "-" ; Hex(mymac(3)) ; "-" ; Hex(mymac(4)) ; "-" ; Hex(mymac(5)) ; "-" ; Hex(mymac(6))
#endif

#if Wachhund = 1
   Reset Watchdog
#endif
Waitms 200                                                  'some routers need time to recognize the NET-IO


#if Network = 1
   If Pinb.1 = 1 Then
     Locate 5 , 1
     Lcd "IP-Adresse automatisch beziehen (DHCP)"
     Call Send_dhcp_discover                                'DHCP request
   Else
     Locate 5 , 1
     Lcd "IP ist manuell konfiguriert"
     Dhcp_flag = 0                                          'to get the ntp, we must have already a valid IP and the MAC of the Router
   End If
#endif

#if Wachhund = 1
   Reset Watchdog
#endif

' Hauptmen anzeigen
Cls
Showpic 0 , 0 , Mainmenu

Call Readtemp

Enable Interrupts

'DMX512 starten
Udr = 0                                                     ' Einmalige Ausgabe einer Null um den ersten "Transmission Complete" Interrupt zu generieren

'Don't insert code here, we need the mainloop !











'****************************** MAINLOOP ******************************
Main:
Do
   #if Wachhund = 1
      Reset Watchdog                                        'falls es einen Hnger gibt
   #endif

   #if Network = 1
      ' Prfen, ob per NTP eine neue Zeit abgerufen werden soll
      If Dhcp_flag = 1 Then
         Call Ntp_get                                       'get the Network Time once
         Dhcp_flag = 0
      End If

      ' Prfen, ob neue Netzwerkdaten vorliegen
      If Data_present = 1 Then
         Call Enc28j60writecontrolregbyte(eie , &H40)       'disable the ENC28J60 int Pin
         Call Enc28j60packetreceive                         'receive the packet
         Data_present = 0
         Call Enc28j60writecontrolregbyte(eie , &HC0)       'enable the ENC28J60 int PIN
      End If
   #endif

   Incr Touchread
   If Touchread = 10000 Then
      Touchread = 0
      ' Touchscreen auslesen
      Call Readtouch
   End If

   ' Folgende Aufgaben jede Sekunde ausfhren
   If Lcd_refresh = 1 And Currentmenu < 7 Then
      Reset Lcd_refresh

      ' Helligkeit erfassen
      Brightness = Getadc(4)                                ' Helligkeitssensor
      Brightness = Brightness / 4

      ' RC5-Codes auslesen
      #if Rc5enabled = 1
         Disable Timer0                                     ' Timer0 deaktivieren, um Ausfhrungsgeschwindigkeit zu erhhen
         Reset Newrc5command

         If Pina.5 = 0 Then
            Enable Timer0                                   ' Timer0 aktivieren, wenn RC5-Codes gesendet werden
            Set Newrc5command
            Set Backlight
         End If

         If Newrc5command = 1 Then
            Getrc5(rc5address , Rc5command)
            Rc5command = Rc5command And &B01111111          'clear the toggle bit - the toggle bit toggles on each new received command - toggle bit is bit 7. Extended RC5 bit is in bit 6
            Call Proceedrc5
         End If
      #endif

      ' Folgende Aufgaben alle 4 1/4 Minuten ausfhren ([Seconds]=255, also 255s/60=4.25min)
      Incr Seconds
      If Seconds = 0 Then
         Reset Backlight                                    ' LCD Hintergrundbeleuchtung ausschalten
         Call Readtemp                                      ' Temperatur neu abfragen !!Achtung: DMX512-Interrupt wird kurzzeitig deaktiviert!!
      End If

      Call Refreshscreen
   End If
Loop
'*********************************** ENDE ************************************
End                                                         'for compiler




'******************************** FUNKTIONEN *********************************

#if Network = 1
   $include "routines28j60.inc"                             ' the ENC28J60-routines
   $include "net_routines.inc"                              ' the net-routines

   ' TCP-Traffic-Filter
   Sub Tcp
      If Tcp_destporth = 0 Then

         If Tcp_destportl = 80 Then                         '80=HTTP
            Call Http
         End If

         If Tcp_destportl = 23 Then                         '23=TELNET
   '         Call Telnet
         End If
      End If
   End Sub

   ' UDP-Traffic-Filter
   Sub Udp_receive
      If T_udp_destport0 = 0 And T_udp_destport1 = 123 Then '123 port we use for the request
         If T_udp_srcport0 = 0 And T_udp_srcport1 = 123 Then       '123 NTP protocol
            Call Ntp
         End If
         Exit Sub
      End If

      If T_udp_destport0 = 0 And T_udp_destport1 = 68 Then  '68 our DHCP port
         If T_udp_srcport0 = 0 And T_udp_srcport1 = 67 Then '67 server DHCP port
            Call Dhcp
         End If
         Exit Sub
      End If
   End Sub
#endif

'********************************  INCLUDINGS ********************************
#if Network = 1
   $include "http.inc"                                      ' HTTP-Server
   $include "tcp.inc"                                       ' TCP-Verarbeitung
   $include "ntp.inc"                                       ' NTP-Verarbeitung
   $include "dhcp.inc"                                      ' DHCP-Verarbeitung
   $include "udpreceive.inc"                                ' Verarbeitung von UDP-Paketen
#endif

$include "isr.inc"                                          ' Interrupt-Service-Routinen
$include "functions.inc"                                    ' Funktionen fr GUI und Zusatzfeatures die nicht das Netzwerk betreffen

'**********************************  BILDER **********************************
Picstartup:
$bgf "picstartup.bgf"
Mainmenu:
$bgf "mainmenu.bgf"
Submenu:
$bgf "submenu.bgf"
Szenenmenu:
$bgf "szenenmenu.bgf"